@@ -9,23 +9,30 @@ from html.parser import HTMLParser
 import md4c
 import polib
 
-from mdpo.command import search_html_command
+from mdpo.command import parse_mdpo_html_command
 from mdpo.html import get_html_attrs_tuple_attr, html_attrs_tuple_to_string
 from mdpo.io import filter_paths, to_file_content_if_is_file
+from mdpo.polib import *  # noqa
 
 
-PROCESS_REPLACER_TAGS = ['p', 'li', 'h1', 'h2', 'h3',
-                         'h4', 'h5', 'h6', 'td', 'th']
+PROCESS_REPLACER_TAGS = [
+    'p', 'li', 'h1', 'h2', 'h3',
+    'h4', 'h5', 'h6', 'td', 'th',
+]
 ALIGNMENT_CHARS = ['\n', ' ', '\t', '\r']
 
 
 class MdPo2HTML(HTMLParser):
-    def __init__(self, pofiles, ignore=[], merge_adjacent_markups=False,
-                 code_tags=['code'], bold_tags=['b', 'strong'],
-                 italic_tags=['em', 'i'], link_tags=['a'], image_tags=['img'],
-                 ignore_grouper_tags=['div', 'hr']):
-        self.pofiles = [polib.pofile(pofilepath) for pofilepath in
-                        filter_paths(glob.glob(pofiles), ignore_paths=ignore)]
+    def __init__(
+        self, pofiles, ignore=[], merge_adjacent_markups=False,
+        code_tags=['code'], bold_tags=['b', 'strong'],
+        italic_tags=['em', 'i'], link_tags=['a'], image_tags=['img'],
+        ignore_grouper_tags=['div', 'hr'],
+    ):
+        self.pofiles = [
+            polib.pofile(pofilepath) for pofilepath in
+            filter_paths(glob.glob(pofiles), ignore_paths=ignore)
+        ]
         self.output = ''
         self.replacer = []
         self._raw_replacement = ''
@@ -77,7 +84,7 @@ class MdPo2HTML(HTMLParser):
                 for _tag in tags_group:
                     if _tag not in template_tags:
                         continue
-                    regex = r'</%s>\s*<%s>' % (tag, _tag)
+                    regex = fr'</{tag}>\s*<{_tag}>'
                     if regex not in regexes:
                         regexes.append(regex)
 
@@ -87,7 +94,7 @@ class MdPo2HTML(HTMLParser):
         return html
 
     def _remove_lastline_from_output_if_empty(self):
-        split_output = self.output.split("\n")
+        split_output = self.output.split('\n')
         if not split_output[-1]:
             self.output = '\n'.join(split_output)[:-1]
 
@@ -109,29 +116,36 @@ class MdPo2HTML(HTMLParser):
 
                 if handled in self.code_tags:
                     _current_replacement += '`'
-                    raw_html_template += '<%s%s>' % (
+                    raw_html_template += '<{}{}>'.format(
                         handled,
-                        (' ' + html_attrs_tuple_to_string(attrs)
-                         if attrs else '')
+                        (
+                            ' ' + html_attrs_tuple_to_string(attrs)
+                            if attrs else ''
+                        ),
                     )
                 elif handled in self.bold_tags:
                     _current_replacement += '**'
-                    raw_html_template += '<%s%s>' % (
+                    raw_html_template += '<{}{}>'.format(
                         handled,
-                        (' ' + html_attrs_tuple_to_string(attrs)
-                         if attrs else '')
+                        (
+                            ' ' + html_attrs_tuple_to_string(attrs)
+                            if attrs else ''
+                        ),
                     )
                 elif handled in self.italic_tags:
                     _current_replacement += '*'
-                    raw_html_template += '<%s%s>' % (
+                    raw_html_template += '<{}{}>'.format(
                         handled,
-                        (' ' + html_attrs_tuple_to_string(attrs)
-                         if attrs else '')
+                        (
+                            ' ' + html_attrs_tuple_to_string(attrs)
+                            if attrs else ''
+                        ),
                     )
                 elif handled in self.link_tags:
-                    title = get_html_attrs_tuple_attr(attrs, "title")
+                    title = get_html_attrs_tuple_attr(attrs, 'title')
                     _current_link_target += '(%s' % get_html_attrs_tuple_attr(
-                        attrs, "href")
+                        attrs, 'href',
+                    )
                     if title:
                         _current_link_target += ' "%s"' % title
                     _current_link_target += ')'
@@ -151,10 +165,11 @@ class MdPo2HTML(HTMLParser):
                     #     attrs_except_href_title) + '>'
                     raw_html_template += '>'
                 else:
-                    raw_html_template += '<%s%s>' % (
+                    raw_html_template += '<{}{}>'.format(
                         handled,
                         ' ' + html_attrs_tuple_to_string(attrs)
-                        if attrs else '')
+                        if attrs else '',
+                    )
                 _last_start_tag = handled
                 if _last_start_tag == 'code':
                     _inside_code = True
@@ -169,8 +184,9 @@ class MdPo2HTML(HTMLParser):
                 else:
                     raw_html_template += '{}'
                     if _current_link_target:
-                        _current_replacement += '[%s]%s' % (
-                            handled, _current_link_target)
+                        _current_replacement += '[{}]{}'.format(
+                            handled, _current_link_target,
+                        )
                         _current_link_target = ''
                     else:
                         _current_replacement += handled
@@ -189,21 +205,23 @@ class MdPo2HTML(HTMLParser):
                 raw_html_template += '<!--%s-->' % handled
             elif handle == 'startend':
                 if handled in self.image_tags:
-                    _current_replacement += '![%s](%s' % (
-                        get_html_attrs_tuple_attr(attrs, "alt"),
-                        get_html_attrs_tuple_attr(attrs, "src"),
+                    _current_replacement += '![{}]({}'.format(
+                        get_html_attrs_tuple_attr(attrs, 'alt'),
+                        get_html_attrs_tuple_attr(attrs, 'src'),
                     )
-                    title = get_html_attrs_tuple_attr(attrs, "title")
+                    title = get_html_attrs_tuple_attr(attrs, 'title')
                     if title:
                         _current_replacement += ' "%s"' % title
                     _current_replacement += ')'
 
                     raw_html_template += '{}'
                 else:
-                    raw_html_template += '<%s%s/>' % (
+                    raw_html_template += '<{}{}/>'.format(
                         handled,
-                        ((' %s' % html_attrs_tuple_to_string(attrs))
-                         if attrs else '')
+                        (
+                            (' %s' % html_attrs_tuple_to_string(attrs))
+                            if attrs else ''
+                        ),
                     )
 
         _current_replacement = html.unescape(_current_replacement)
@@ -214,7 +232,8 @@ class MdPo2HTML(HTMLParser):
         else:
             if self._current_msgctxt:
                 replacement = self.translations_with_msgctxt[
-                    self._current_msgctxt].get(_current_replacement)
+                    self._current_msgctxt
+                ].get(_current_replacement)
             else:
                 replacement = self.translations.get(_current_replacement)
             if not replacement:
@@ -244,7 +263,8 @@ class MdPo2HTML(HTMLParser):
                 html_before_first_replacement.split('<a href="')[0]
 
         html_inner = '\n'.join(
-            self.html_renderer.parse(replacement).split('\n')[:-1])
+            self.html_renderer.parse(replacement).split('\n')[:-1],
+        )
         html_inner = '>'.join(html_inner.split('>')[1:])
         html_inner = '<'.join(html_inner.split('<')[:-1])
 
@@ -252,8 +272,10 @@ class MdPo2HTML(HTMLParser):
             html_after_last_replacement
 
         if self.merge_adjacent_markups:
-            html_template = self._merge_adyacent_tags(html_template,
-                                                      template_tags)
+            html_template = self._merge_adyacent_tags(
+                html_template,
+                template_tags,
+            )
 
         self.output += html_template
         self.context = []
@@ -269,17 +291,21 @@ class MdPo2HTML(HTMLParser):
 
         if tag in self.ignore_grouper_tags:
             self.context.append(tag)
-            self.output += '<%s%s>' % (
-                tag, ' ' + html_attrs_tuple_to_string(attrs) if attrs else '')
+            self.output += '<{}{}>'.format(
+                tag, ' ' + html_attrs_tuple_to_string(attrs) if attrs else '',
+            )
         elif self.context and self.context[0] in self.ignore_grouper_tags:
-            self.output += '<%s%s>' % (
-                tag, ' ' + html_attrs_tuple_to_string(attrs) if attrs else '')
+            self.output += '<{}{}>'.format(
+                tag, ' ' + html_attrs_tuple_to_string(attrs) if attrs else '',
+            )
         elif tag == 'ul' and not self.context:
-            self.output += '<%s%s>' % (
-                tag, ' ' + html_attrs_tuple_to_string(attrs) if attrs else '')
+            self.output += '<{}{}>'.format(
+                tag, ' ' + html_attrs_tuple_to_string(attrs) if attrs else '',
+            )
         elif tag in ['blockquote', 'table', 'thead', 'tbody', 'tr']:
-            self.output += '<%s%s>' % (
-                tag, ' ' + html_attrs_tuple_to_string(attrs) if attrs else '')
+            self.output += '<{}{}>'.format(
+                tag, ' ' + html_attrs_tuple_to_string(attrs) if attrs else '',
+            )
         else:
             self.replacer.append(('start', tag, attrs))
             self.context.append(tag)
@@ -317,7 +343,8 @@ class MdPo2HTML(HTMLParser):
         if data:
             if not self.replacer or (
                     self.context and
-                    self.context[0] in self.ignore_grouper_tags) or \
+                    self.context[0] in self.ignore_grouper_tags
+            ) or \
                     not self.context:
                 self.output += data
                 return
@@ -333,7 +360,7 @@ class MdPo2HTML(HTMLParser):
             self.replacer.append(('comment', data, None))
         else:
             data_as_comment = '<!--%s-->' % data
-            command, comment = search_html_command(data_as_comment)
+            command, comment = parse_mdpo_html_command(data_as_comment)
             if command is None:
                 self.output += data_as_comment
             else:
@@ -347,11 +374,13 @@ class MdPo2HTML(HTMLParser):
                 elif command == 'enable-next-line':
                     self._enable_next_line = True
                 elif command == 'context' and comment:
-                    self._current_msgctxt = comment.strip(" ")
+                    self._current_msgctxt = comment.strip(' ')
                 elif command == 'include-codeblock':
-                    warnings.warn("Code blocks translations is not supported"
-                                  " by mdpo2html implementation.",
-                                  SyntaxWarning)
+                    warnings.warn(
+                        'Code blocks translations is not supported'
+                        ' by mdpo2html implementation.',
+                        SyntaxWarning,
+                    )
 
     def translate(self, filepath_or_content, save=None):
         content = to_file_content_if_is_file(filepath_or_content)
@@ -364,14 +393,15 @@ class MdPo2HTML(HTMLParser):
                     if entry.msgctxt not in self.translations_with_msgctxt:
                         self.translations_with_msgctxt[entry.msgctxt] = {}
                     self.translations_with_msgctxt[
-                        entry.msgctxt][entry.msgid] = entry.msgstr
+                        entry.msgctxt
+                    ][entry.msgid] = entry.msgstr
                 else:
                     self.translations[entry.msgid] = entry.msgstr
 
         self.feed(content)
 
         if save:
-            with open(save, "w") as f:
+            with open(save, 'w') as f:
                 f.write(self.output)
 
         self.reset()
@@ -379,8 +409,10 @@ class MdPo2HTML(HTMLParser):
         return self.output
 
 
-def markdown_pofile_to_html(filepath_or_content, pofiles, ignore=[],
-                            save=None, **kwargs):
+def markdown_pofile_to_html(
+    filepath_or_content, pofiles, ignore=[],
+    save=None, **kwargs,
+):
     """Produces a translated HTML file given a previous HTML file (created by a
     Markdown-to-HTML processor) and a set of pofiles as reference for msgstrs.
 
@@ -404,5 +436,5 @@ def markdown_pofile_to_html(filepath_or_content, pofiles, ignore=[],
         str: HTML output translated version of the given file.
     """
     return MdPo2HTML(
-        pofiles, ignore=ignore, **kwargs
+        pofiles, ignore=ignore, **kwargs,
     ).translate(filepath_or_content, save=save)
